/* 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */                          

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.text.*;
import com.sun.java.accessibility.*;


/**
 * <P>Explorer is an example assistive technology that uses EventQueueMonitor,
 *
 * <P>For now, it is meant to run in the same Java Virtual Machine as 
 * the application it is accessing.  To do this, you need to make sure
 * there is an "AWT.AutoLoadClasses=" line in the file,
 * $JDKHOME/lib/awt.properties.  For example, you would add the following 
 * line in awt.properties to make this thread run with the application:
 * <PRE>
 * AWT.AutoLoadClasses=Explorer
 * </PRE>
 * <P>Note that AWT.AutoLoadClasses can take a comma-separated list of 
 * assistive technology classes.  Note also that you need to make sure 
 * the classes for the assistive technologies are in your CLASSPATH
 * environment variable.
 *
 * <P>Furthermore, this class is dynamically loaded and a single instance
 * is created by the event queue class (see EventQueueMonitor) in a separate 
 * thread.  For this to work properly, the constructor method needs to 
 * do all the work.
 *
 * <P>For more information on the event delegation model, see the event
 * delegation model specification that comes with the JDK 1.1.1 documentation.
 *
 * @see EventQueueMonitor
 *
 * @version     1.0 08/02/97 17:08:39
 * @author      Peter Korn
 */
public class Explorer extends Frame
    implements MouseMotionListener, ActionListener, MenuListener {
    
    List mouseInformation;
    TimeTracker tracker;
    Timer timer;
    JMenu activeMenu = null; 

    // Create the GUI
    //
    public Explorer() {
        super("Explorer");
	    
        // Make the menu bar
        //
        MenuBar mb  = new MenuBar();
        Menu m      = new Menu("File");
        MenuItem mi = new MenuItem("Exit");
        mi.addActionListener(this);
        m.add(mi);
        mb.add(m);

       setMenuBar(mb);


        // Add Mouse Info List
        //
	    mouseInformation = new List(8);
		mouseInformation.setBounds(0, 0, 150, 150);
        add(mouseInformation);

        pack();
        show();

        // Create a 1/2 sec. delay timer
        //
//        tracker = new TimeTracker(500, this);
//        tracker.timer.stop();   // just being sure...

        timer = new Timer(500, this);
        timer.stop();   // just being sure...

	SwingEventMonitor.addMouseMotionListener((MouseMotionListener)this);
	SwingEventMonitor.addMenuListener((MenuListener)this);
    }
    
    public void actionPerformed(ActionEvent e) {
        String s = e.getActionCommand();
        if (s == "Exit") {
            System.exit(0);
        } else {
            timerTimeout();
        }
    }  

    // Display Accessible information about the object under the mouse
    //
    public void displayInfo(Accessible information, Point p) {
        mouseInformation.setVisible(false);
        mouseInformation.removeAll();
        if (information == null) {
            mouseInformation.addItem("(null)");
        } else {
	    mouseInformation.addItem("Accessible information:");
            mouseInformation.addItem("    Name:  " + information.getAccessibleName());
            mouseInformation.addItem("    Desc:  " + information.getAccessibleDescription());
            mouseInformation.addItem("    Role:  " + information.getAccessibleRole());
            mouseInformation.addItem("    State(s): " + information.getAccessibleStateSet().toString());
            mouseInformation.addItem("    Value: " + information.getAccessibleValue());

            Accessible parent = information.getAccessibleParent();
            if (parent != null) {
                mouseInformation.addItem("    Parent: " + parent.getAccessibleName());
            }
            mouseInformation.addItem("    # Children: " + information.getAccessibleChildrenCount());
	    Accessible sibling = information.getNextAccessibleSibling();
	    if (sibling != null) {
            	mouseInformation.addItem("    Next Sibling: " + sibling.getAccessibleName());
            }
	    sibling = information.getPreviousAccessibleSibling();
	    if (sibling != null) {
                mouseInformation.addItem("    Previous Sibling: " + sibling.getAccessibleName());
	    }
            mouseInformation.addItem("    # Actions: " + information.getAccessibleActionCount());
            if (information.getAccessibleActionCount() > 0) {
                mouseInformation.addItem("    Default Action: " + information.getAccessibleActionDescription(1));
            }
	    mouseInformation.addItem("    # Things Selected: " + information.getAccessibleSelectionCount());

	    mouseInformation.addItem("Accessible information, from awt.Component:");
            mouseInformation.addItem("    Background Color: " + information.getBackground().toString());
            mouseInformation.addItem("    Foreground Color: " + information.getForeground().toString());
            mouseInformation.addItem("    Cursor: " + information.getCursor().toString());
            mouseInformation.addItem("    Font: " + information.getFont().toString());
            mouseInformation.addItem("    Font Metrics: " + information.getFontMetrics(information.getFont()).toString());
            mouseInformation.addItem("    Enabled: " + information.isEnabled());
            mouseInformation.addItem("    Visible: " + information.isVisible());
            mouseInformation.addItem("    Showing: " + information.isShowing());
	    if (information.isShowing()) {
                mouseInformation.addItem("    Location on Screen: " + information.getLocationOnScreen().toString());
	    }
            mouseInformation.addItem("    Bounds: " + information.getBounds().toString());
            mouseInformation.addItem("    Size: " + information.getSize().toString());
            mouseInformation.addItem("    Focus Traversable: " + information.isFocusTraversable());
	    mouseInformation.addItem("    Locale: " + information.getLocale().toString());

            AccessibleText textInfo = information.getAccessibleText();
            if (textInfo != null) {
		mouseInformation.addItem("Accessible Text information:");
                int i = textInfo.getIndexAtPoint(p);
		mouseInformation.addItem("    Index under mouse: " + i);
                Object o;
                o = textInfo.getAtIndex(AccessibleText.CHARACTER, i);
                if (o instanceof String) { 
                    mouseInformation.addItem("    Character under mouse: " + (String) o);
                }
                o = textInfo.getAtIndex(AccessibleText.WORD, i);
                if (o instanceof String) {           
                    mouseInformation.addItem("    Word under mouse:" +  (String) o);
                }
                o = textInfo.getAtIndex(AccessibleText.SENTENCE, i);
                if (o instanceof String) {           
                    mouseInformation.addItem("    Sentence under mouse: " + (String) o);
                }
		mouseInformation.addItem("    Char/Object rect under mouse: " + textInfo.getCharacterBounds(i));
		mouseInformation.addItem("    Char count of object: " + textInfo.getCharCount());
		mouseInformation.addItem("    Caret position: " + textInfo.getCaretPosition());
		AttributeSet attributes = textInfo.getCharacterAttribute(i);
		Enumeration attrList = attributes.getAttributeNames();
		String attrText = new String("");
		while (attrList.hasMoreElements()) {
		    attrText += (String) attrList.nextElement();
		    if (attrList.hasMoreElements()) {
			attrText += ", ";
		    }
		}
		mouseInformation.addItem("    Char attributes under mouse: " + attrText);
		int start = textInfo.getSelectionStart();
		int end = textInfo.getSelectionEnd();
		if (start != end) {
		    mouseInformation.addItem("    Selection range: [" +
		    start + ", " + end + "]");
		    Object selection = textInfo.getSelectedText();
		    if (selection instanceof String) {
		        mouseInformation.addItem("    Selected text: " + (String) selection);
		    }
		}
	    }
        }
        mouseInformation.setVisible(true);
    }

    // Handle mouse movements
    //
    public void mouseDragged(MouseEvent e) {
//        tracker.timer.stop();
//        tracker.timer.start();
        timer.stop();
        timer.start();
    }

    public void mouseMoved(MouseEvent e) {
//        tracker.timer.stop();
//        tracker.timer.start();
        timer.stop();
        timer.start();
    }

    public void menuCancelled(MenuEvent theEvent) {
        activeMenu = null;
    }

    public void menuDeselected(MenuEvent theEvent) {
        activeMenu = null;
    }

    public void menuSelected(MenuEvent theEvent) {
        activeMenu = (JMenu) theEvent.getSource();
    }

    public void timerTimeout() {
//        tracker.timer.stop();
        timer.stop();
        Point currentMousePos = EventQueueMonitor.getCurrentMousePosition();
        Accessible a = null;
  	if (activeMenu != null) {
	    JPopupMenu pm = activeMenu.getPopupMenu();
            Point menuLoc = pm.getLocationOnScreen();
            Point menuPoint = new Point(currentMousePos.x - menuLoc.x,
                                        currentMousePos.y - menuLoc.y);
            Component c = pm.getComponentAt(menuPoint);
	    if (c instanceof Accessible) {
		a = (Accessible) c;
	    }
	}
	if (a == null) {
	    a = EventQueueMonitor.getAccessibleAt(currentMousePos);
	}
        if (a != null) {
            Point containerLoc = a.getLocationOnScreen();
            Point containerPoint = new Point(currentMousePos.x - containerLoc.x,
						 currentMousePos.y - containerLoc.y);
            displayInfo(a, containerPoint);
        } else {
	    displayInfo(null, null);
	}
    }

   static public void main(String args[]) {
        new Explorer();
    }

}

/**
 *  Wrapper class around Timer to recieve ActionPerformed's
 */
class TimeTracker implements ActionListener {
    public Timer timer;
    Explorer explorer; 

    public TimeTracker(int delay, Explorer e) {
        explorer = e;
        timer = new Timer(delay, this);
    }

    public void actionPerformed(ActionEvent e) {
        explorer.timerTimeout();
    }  
}



