/*
 * @(#)JavaMonitor.java	1.20 97/10/08
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

import java.util.Properties;
import java.beans.*;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.text.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.undo.*;
import com.sun.java.accessibility.*;

/**
 * <P>JavaMonitor is an example assistive technology that uses EventQueueMonitor,
 * AWTEventMonitor, and SwingEventMonitor.
 *
 * <P>For now, it is meant to run in the same Java Virtual Machine as 
 * the application it is accessing.  To do this, you need to make sure
 * there is an "AWT.AutoLoadClasses=" line in the file,
 * $JDKHOME/lib/awt.properties.  For example, you would add the following 
 * line in awt.properties to make this thread run with the application:
 * <PRE>
 * AWT.AutoLoadClasses=JavaMonitor
 * </PRE>
 * <P>Note that AWT.AutoLoadClasses can take a comma-separated list of 
 * assistive technology classes.  Note also that you need to make sure 
 * the classes for the assistive technologies are in your CLASSPATH
 * environment variable.
 *
 * <P>Furthermore, this class is dynamically loaded and a single instance
 * is created by the event queue class (see EventQueueMonitor) in a separate 
 * thread.  For this to work properly, the constructor method needs to 
 * do all the work.
 *
 * <P>For more information on the event delegation model, see the event
 * delegation model specification that comes with the JDK 1.1.1 documentation.
 *
 * <p>NOTE:  This is a preliminary draft.  The methods and name may change 
 * in future beta releases.
 *
 * @see EventQueueMonitor
 * @see AWTEventMonitor
 * @see SwingEventMonitor
 *
 * @version     1.20 10/08/97 13:30:37
 * @author      Willie Walker
 * @author      Peter Korn
 */
public class JavaMonitor extends Frame {
    JavaMonitorEventListener jmel = null;
    GridBagLayout gbl = new GridBagLayout();

    protected JMenu activeMenu = null;

    List eventStatusList;
    int  eventStatusListSize = 0;

    List focusInformation;
    List mouseInformation;

    // Create the GUI
    //
    public JavaMonitor() {
        super("Java Monitor");
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gbl);

        // Make the menu bar
        //
        MenuBar mb = new MenuBar();
        Menu m     = new Menu("File");
        m.add("Exit");
        mb.add(m);
        setMenuBar(mb);

        // Make the event selector checkbox area
        //
        jmel = new JavaMonitorEventListener(this);

        Checkbox cb;
        Panel eventSelector = new Panel();
        eventSelector.setLayout(new GridLayout(4,6));
        eventSelector.add(cb = new Checkbox("Action"));
        cb.addItemListener(new JavaMonitorCommand(EventID.ACTION,jmel));

        eventSelector.add(cb = new Checkbox("Adjustment"));
        cb.addItemListener(new JavaMonitorCommand(EventID.ADJUSTMENT,jmel));

        eventSelector.add(cb = new Checkbox("Component",false));
        cb.addItemListener(new JavaMonitorCommand(EventID.COMPONENT,jmel));

        eventSelector.add(cb = new Checkbox("Container",false));
        cb.addItemListener(new JavaMonitorCommand(EventID.CONTAINER,jmel));

        eventSelector.add(cb = new Checkbox("Focus"));
        cb.addItemListener(new JavaMonitorCommand(EventID.FOCUS,jmel));

        eventSelector.add(cb = new Checkbox("Item"));
        cb.addItemListener(new JavaMonitorCommand(EventID.ITEM,jmel));

        eventSelector.add(cb = new Checkbox("Key"));
        cb.addItemListener(new JavaMonitorCommand(EventID.KEY,jmel));

        eventSelector.add(cb = new Checkbox("Mouse"));
        cb.addItemListener(new JavaMonitorCommand(EventID.MOUSE,jmel));

        eventSelector.add(cb = new Checkbox("Mouse Motion"));
        cb.addItemListener(new JavaMonitorCommand(EventID.MOTION,jmel));

        eventSelector.add(cb = new Checkbox("Paint/Update"));
        cb.addItemListener(new JavaMonitorCommand(EventID.PAINT,jmel));

        eventSelector.add(cb = new Checkbox("Text"));
        cb.addItemListener(new JavaMonitorCommand(EventID.TEXT,jmel));

        eventSelector.add(cb = new Checkbox("Window",false));
        cb.addItemListener(new JavaMonitorCommand(EventID.WINDOW,jmel));

//        eventSelector.add(cb = new Checkbox("JAncestor"));
//        cb.addItemListener(new JavaMonitorCommand(EventID.JANCESTOR,jmel));

        eventSelector.add(cb = new Checkbox("JCellEditor"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JCELLEDITOR,jmel));

        eventSelector.add(cb = new Checkbox("JChange"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JCHANGE,jmel));

        eventSelector.add(cb = new Checkbox("JDocument"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JDOCUMENT,jmel));

        eventSelector.add(cb = new Checkbox("JListData"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JLISTDATA,jmel));

        eventSelector.add(cb = new Checkbox("JListSelection")); 
        cb.addItemListener(new JavaMonitorCommand(EventID.JLISTSELECTION,jmel));

        eventSelector.add(cb = new Checkbox("JMenu"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JMENU,jmel));

        eventSelector.add(cb = new Checkbox("JPropertyChange"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JPROPERTYCHANGE,jmel));

        eventSelector.add(cb = new Checkbox("JColumnModel"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JCOLUMNMODEL,jmel));

        eventSelector.add(cb = new Checkbox("JTableModel"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JTABLEMODEL,jmel));

        eventSelector.add(cb = new Checkbox("JTreeExpansion"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JTREEEXPANSION,jmel));

        eventSelector.add(cb = new Checkbox("JTreeModel"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JTREEMODEL,jmel));

        eventSelector.add(cb = new Checkbox("JTreeSelection")); 
        cb.addItemListener(new JavaMonitorCommand(EventID.JTREESELECTION,jmel));

        eventSelector.add(cb = new Checkbox("JUndoableEdit"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JUNDOABLEEDIT,jmel));

        eventSelector.add(cb = new Checkbox("JVetoableChange"));
        cb.addItemListener(new JavaMonitorCommand(EventID.JVETOABLECHANGE,jmel));

        addComp(this,eventSelector,0,0,2,1,0.0,0.0,
                new Insets(4,4,0,4));

        // Make the event status list
        //
        addComp(this,eventStatusList = new List(10,true),0,1,2,1,1.0,1.0,
                new Insets(4,4,0,4));

        // Add the focus information
        //
        addComp(this,new Label("Last Focus Detail:"),0,2,1,1,0.5,0.0,
                new Insets(4,4,0,0));
        addComp(this,focusInformation = new List(8),0,3,1,1,0.5,0.0,
                new Insets(0,4,4,4));
                

        // Add the mouse information
        //
        addComp(this,new Label("Object under Mouse (F1):"),1,2,1,1,0.5,0.0,
                new Insets(4,0,0,0));
        addComp(this,mouseInformation = new List(8),1,3,1,1,0.5,0.0,
                new Insets(0,0,4,4));

        pack();
        show();
    }

    static void addComp(Container cont, Component comp,
                        int x, int y,
                        int w, int h,
                        double weightx, double weighty,
                        Insets insets) {

        cont.add(comp);

        GridBagLayout gbl = (GridBagLayout)cont.getLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = weightx;
        c.weighty = weighty;
        c.insets = insets;
        gbl.setConstraints(comp,c);
    }

    public void displayInformationInfo(List list, Accessible information, Point p) {
        list.removeAll();
        if (information == null) {
            list.addItem("(null)");
        } else {
            list.addItem("Name: " + information.getAccessibleName());
            list.addItem("Desc: " + information.getAccessibleDescription());
            list.addItem("Role: " + information.getAccessibleRole());
            list.addItem("State: " + information.getAccessibleStateSet().toString());
            list.addItem("Value: " + information.getAccessibleValue());

            AccessibleText textInfo = information.getAccessibleText();
            if (textInfo != null) {
                int i = textInfo.getIndexAtPoint(p);
                Object o;
                o = textInfo.getAtIndex(AccessibleText.CHARACTER, i);
                if (o instanceof String) {
                    mouseInformation.addItem("Character: " + (String) o);
                }
                o = textInfo.getAtIndex(AccessibleText.WORD, i);
                if (o instanceof String) {
                    mouseInformation.addItem("Word: " +  (String) o);
		}
                o = textInfo.getAtIndex(AccessibleText.SENTENCE, i);
                if (o instanceof String) {
                    mouseInformation.addItem("Sentence: " + (String) o);
                }
	    }
        }
    }

    public void displayFocusInformation() {
        Accessible a;
        a = Translator.getAccessible(SwingEventMonitor.getComponentWithFocus());
        displayInformationInfo(focusInformation,a,null);
    }

    public void displayMouseInformation(Accessible information, Point p) {
        displayInformationInfo(mouseInformation,information,p);
    }

    public void displayEvent(String eventString) {
        if (eventStatusListSize > 100) {
            eventStatusList.delItem(0);
        } else {
            eventStatusListSize++;
        }

        eventStatusList.addItem(eventString);
        eventStatusList.makeVisible(eventStatusList.getItemCount() - 1);
    }

    static public void main(String args[]) {
        new JavaMonitor();
    }
}

class JavaMonitorCommand implements ItemListener {
    int    id;
    JavaMonitorEventListener jmel;

    public JavaMonitorCommand(int id, JavaMonitorEventListener jmel) {
        this.id   = id;
        this.jmel = jmel;
    }

    public void itemStateChanged(ItemEvent e) {
        if (e.getStateChange() == ItemEvent.SELECTED) {
            switch (id) {

            case EventID.COMPONENT:
                SwingEventMonitor.addComponentListener((ComponentListener)jmel);
                break;

            case EventID.CONTAINER:
                SwingEventMonitor.addContainerListener((ContainerListener)jmel);
                break;

            case EventID.FOCUS:
                SwingEventMonitor.addFocusListener((FocusListener)jmel);
                break;

            case EventID.KEY:
                SwingEventMonitor.addKeyListener((KeyListener)jmel);
                break;

            case EventID.MOUSE:
                SwingEventMonitor.addMouseListener((MouseListener)jmel);
                break;

            case EventID.MOTION:
                SwingEventMonitor.addMouseMotionListener((MouseMotionListener)jmel);
                break;
                
            case EventID.WINDOW:
                SwingEventMonitor.addWindowListener((WindowListener)jmel);
                break;
               
            case EventID.ACTION:
                SwingEventMonitor.addActionListener((ActionListener)jmel);
                break;
                
            case EventID.ADJUSTMENT:
                SwingEventMonitor.addAdjustmentListener((AdjustmentListener)jmel);
                break;
                
            case EventID.ITEM:
                SwingEventMonitor.addItemListener((ItemListener)jmel);
                break;

            case EventID.TEXT:
                SwingEventMonitor.addTextListener((TextListener)jmel);
                break;

//            case EventID.JANCESTOR:
//                SwingEventMonitor.addAncestorListener((AncestorListener)jmel);
//                break;

            case EventID.JCELLEDITOR:
                SwingEventMonitor.addCellEditorListener((CellEditorListener)jmel);
                break;

            case EventID.JCHANGE:
                SwingEventMonitor.addChangeListener((ChangeListener)jmel);
                break;

            case EventID.JDOCUMENT:
                SwingEventMonitor.addDocumentListener((DocumentListener)jmel);
                break;

            case EventID.JLISTDATA:
                SwingEventMonitor.addListDataListener((ListDataListener)jmel);
                break;

            case EventID.JLISTSELECTION:
                SwingEventMonitor.addListSelectionListener((ListSelectionListener)jmel);
                break;

            case EventID.JMENU:
                SwingEventMonitor.addMenuListener((MenuListener)jmel);
                break;

            case EventID.JPROPERTYCHANGE:
                SwingEventMonitor.addPropertyChangeListener((PropertyChangeListener)jmel);
                break;

            case EventID.JCOLUMNMODEL:
                SwingEventMonitor.addColumnModelListener((TableColumnModelListener)jmel);
                break;

            case EventID.JTABLEMODEL:
                SwingEventMonitor.addTableModelListener((TableModelListener)jmel);
                break;

            case EventID.JTREEEXPANSION:
                SwingEventMonitor.addTreeExpansionListener((TreeExpansionListener)jmel);
                break;

            case EventID.JTREEMODEL:
                SwingEventMonitor.addTreeModelListener((TreeModelListener)jmel);
                break;

            case EventID.JTREESELECTION:
                SwingEventMonitor.addTreeSelectionListener((TreeSelectionListener)jmel);
                break;

            case EventID.JUNDOABLEEDIT:
                SwingEventMonitor.addUndoableEditListener((UndoableEditListener)jmel);
                break;

            case EventID.JVETOABLECHANGE:
                SwingEventMonitor.addVetoableChangeListener((VetoableChangeListener)jmel);
                break;

            default:
                break;
            }
        } else {
            switch (id) {

            case EventID.COMPONENT:
                SwingEventMonitor.removeComponentListener((ComponentListener)jmel);
                break;

            case EventID.CONTAINER:
                SwingEventMonitor.removeContainerListener((ContainerListener)jmel);
                break;

            case EventID.FOCUS:
                SwingEventMonitor.removeFocusListener((FocusListener)jmel);
                break;

            case EventID.KEY:
                SwingEventMonitor.removeKeyListener((KeyListener)jmel);
                break;

            case EventID.MOUSE:
                SwingEventMonitor.removeMouseListener((MouseListener)jmel);
                break;

            case EventID.MOTION:
                SwingEventMonitor.removeMouseMotionListener((MouseMotionListener)jmel);
                break;

            case EventID.WINDOW:
                SwingEventMonitor.removeWindowListener((WindowListener)jmel);
                break;

            case EventID.ACTION:
                SwingEventMonitor.removeActionListener((ActionListener)jmel);
                break;

            case EventID.ADJUSTMENT:
                SwingEventMonitor.removeAdjustmentListener((AdjustmentListener)jmel);
                break;

            case EventID.ITEM:
                SwingEventMonitor.removeItemListener((ItemListener)jmel);
                break;

            case EventID.TEXT:
                SwingEventMonitor.removeTextListener((TextListener)jmel);
                break;

//            case EventID.JANCESTOR:
//                SwingEventMonitor.removeAncestorListener((AncestorListener)jmel);
//                break;

            case EventID.JCELLEDITOR:
                SwingEventMonitor.removeCellEditorListener((CellEditorListener)jmel);
                break;

            case EventID.JCHANGE:
                SwingEventMonitor.removeChangeListener((ChangeListener)jmel);
                break;

            case EventID.JDOCUMENT:
                SwingEventMonitor.removeDocumentListener((DocumentListener)jmel);
                break;

            case EventID.JLISTDATA:
                SwingEventMonitor.removeListDataListener((ListDataListener)jmel);
                break;

            case EventID.JLISTSELECTION:
                SwingEventMonitor.removeListSelectionListener((ListSelectionListener)jmel);
                break;

            case EventID.JMENU:
                SwingEventMonitor.removeMenuListener((MenuListener)jmel);
                break;

            case EventID.JPROPERTYCHANGE:
                SwingEventMonitor.removePropertyChangeListener((PropertyChangeListener)jmel);
                break;

            case EventID.JCOLUMNMODEL:
                SwingEventMonitor.removeColumnModelListener((TableColumnModelListener)jmel);
                break;

            case EventID.JTREEEXPANSION:
                SwingEventMonitor.removeTreeExpansionListener((TreeExpansionListener)jmel);
                break;

            case EventID.JTREEMODEL:
                SwingEventMonitor.removeTreeModelListener((TreeModelListener)jmel);
                break;

            case EventID.JTREESELECTION:
                SwingEventMonitor.removeTreeSelectionListener((TreeSelectionListener)jmel);
                break;

            case EventID.JUNDOABLEEDIT:
                SwingEventMonitor.removeUndoableEditListener((UndoableEditListener)jmel);
                break;

            case EventID.JVETOABLECHANGE:
                SwingEventMonitor.removeVetoableChangeListener((VetoableChangeListener)jmel);
                break;

            default:
                break;
            }
        }
    }
}

class JavaMonitorEventListener implements ComponentListener, ContainerListener,
    FocusListener, KeyListener, MouseListener, MouseMotionListener,
    WindowListener, ActionListener,AdjustmentListener,ItemListener,
    TextListener, AncestorListener, CellEditorListener, ChangeListener, 
    DocumentListener, ListDataListener, ListSelectionListener, 
    MenuListener, PropertyChangeListener, TableColumnModelListener, 
    TableModelListener, TreeExpansionListener, TreeModelListener, 
    TreeSelectionListener, UndoableEditListener, VetoableChangeListener  {

    JavaMonitor app;

    public JavaMonitorEventListener(JavaMonitor app) {
        this.app = app;
    }

        /* ActionListener Methods ***************************************/
  
    public void actionPerformed(ActionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* AdjustmentListener Methods ***********************************/

    public void adjustmentValueChanged(AdjustmentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* ItemListener Methods *****************************************/

    public void itemStateChanged(ItemEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

       /* ComponentListener Methods ************************************/
  
    public void componentMoved(ComponentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void componentResized(ComponentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void componentShown(ComponentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void componentHidden(ComponentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

       /* ContainerListener Methods ************************************/

    public void componentAdded(ContainerEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void componentRemoved(ContainerEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* FocusListener Methods ****************************************/
  
    public void focusGained(FocusEvent theEvent) {
        app.displayFocusInformation();
        app.displayEvent(theEvent.toString());
    }

    public void focusLost(FocusEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* KeyListener Methods ******************************************/
  
    public void keyTyped(KeyEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void keyPressed(KeyEvent theEvent) {
        String statusString = null;
        if (theEvent.getKeyCode() == KeyEvent.VK_F1) {
            Point currentMousePos = EventQueueMonitor.getCurrentMousePosition();
	    Accessible a = null;
  	    if (app.activeMenu != null) {
		JPopupMenu pm = app.activeMenu.getPopupMenu();
                Point menuLoc = pm.getLocationOnScreen();
                Point menuPoint = new Point(currentMousePos.x - menuLoc.x,
                                            currentMousePos.y - menuLoc.y);
                Component c = pm.getComponentAt(menuPoint);
	        if (c instanceof Accessible) {
		    a = (Accessible) c;
	        }
	    }
	    if (a == null) {
	        a = EventQueueMonitor.getAccessibleAt(currentMousePos);
	    }
	    Point containerLoc = a.getLocationOnScreen();
	    Point containerPoint = new Point(currentMousePos.x - containerLoc.x,
					     currentMousePos.y - containerLoc.y);
            app.displayMouseInformation(a, containerPoint);
        }
        app.displayEvent(theEvent.toString());
    }

    public void keyReleased(KeyEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* MouseListener Methods ****************************************/
  
    public void mouseClicked(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void mousePressed(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void mouseReleased(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void mouseEntered(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void mouseExited(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* MouseMotionListener Methods **********************************/
  
    public void mouseDragged(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void mouseMoved(MouseEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowOpened(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* TextListener Methods *****************************************/
  
    public void textValueChanged(TextEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
  
        /* WindowListener Methods ***************************************/
  
    public void windowClosing(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowClosed(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowIconified(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowDeiconified(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowActivated(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void windowDeactivated(WindowEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* AncestorListener Methods ******************************************/

    public void ancestorAdded(AncestorEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void ancestorRemoved(AncestorEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void ancestorMoved(AncestorEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

        /* CellEditorListener Methods *****************************************/

    public void editingStarted(ChangeEvent theEvent) {
	app.displayEvent(theEvent.toString());
    }

    public void editingStopped(ChangeEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void editingCanceled(ChangeEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* ChangeListener Methods *****************************************/

    public void stateChanged(ChangeEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
    

        /* DocumentListener Methods **************************************/
  
    public void changedUpdate(DocumentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void insertUpdate(DocumentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void removeUpdate(DocumentEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
    

        /* ListSelectionListener Methods ***********************************/
  
    public void valueChanged(ListSelectionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
    

        /* ListDataListener Methods *****************************************/
  
    public void contentsChanged(ListDataEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void intervalAdded(ListDataEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void intervalRemoved(ListDataEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* MenuListener Methods *****************************************/
 
    public void menuCancelled(MenuEvent theEvent) {
        app.activeMenu = null;
        app.displayEvent(theEvent.toString());
    }

    public void menuDeselected(MenuEvent theEvent) {
        app.activeMenu = null;
        app.displayEvent(theEvent.toString());
    }

    public void menuSelected(MenuEvent theEvent) {
        app.activeMenu = (JMenu) theEvent.getSource();
        app.displayEvent(theEvent.toString());
    }


        /* PropertyChangeListener Methods **********************************/
  
    public void propertyChange(PropertyChangeEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* TableColumnModelListener Methods *******************************/
 
    public void columnAdded(TableColumnModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void columnMarginChanged(ChangeEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void columnMoved(TableColumnModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void columnRemoved(TableColumnModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void columnSelectionChanged(ListSelectionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

 
        /* TableModelListener Methods **************************************/

    public void tableChanged(TableModelEvent theEvent) {
	app.displayEvent(theEvent.toString());
    }

    public void tableRowsInserted(TableModelEvent theEvent) {
	app.displayEvent(theEvent.toString());
    }

    public void tableRowsRemoved(TableModelEvent theEvent) {
	app.displayEvent(theEvent.toString());
    }

        /* TreeExpansionListener Methods **********************************/
  
    public void treeCollapsed(TreeExpansionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
  
    public void treeExpanded(TreeExpansionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

  
        /* TreeModelListener Methods **********************************/
 
    public void treeNodesChanged(TreeModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
  
    public void treeNodesInserted(TreeModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }
  
    public void treeNodesRemoved(TreeModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }

    public void treeStructureChanged(TreeModelEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }  

        /* TreeSelectionListener Methods ***********************************/
  
    public void valueChanged(TreeSelectionEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* UndoableEditListener Methods **************************************/

    public void undoableEditHappened(UndoableEditEvent theEvent) {
        app.displayEvent(theEvent.toString());
    }


        /* VetoableChangeListener Methods **********************************/

    public void vetoableChange(PropertyChangeEvent theEvent) 
	    throws PropertyVetoException {
        app.displayEvent(theEvent.toString());
    }
}
