/*
 * @(#)Monkey.java	1.6 97/09/26
 * 
 * Copyright (c) 1997 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 */

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import com.sun.java.swing.*;
import com.sun.java.swing.tree.*;
import com.sun.java.swing.event.*;
import com.sun.java.accessibility.*;

/**
 * <P>Monkey is an example assistive technology that uses the EventQueueMonitor
 * utility class from com.sun.java.accessibility.  Like a Monkey, it runs 
 * around in trees.  In particular, it runs around the Component tree in an 
 * application running in the same VM as the Monkey.  On the left hand side 
 * of Monkey is a tree that represents the Component hierarchy.  On the right 
 * hand side of Monkey is the same tree, but viewed as though the components 
 * are Accessible objects.  In order to tell Monkey to learn about an updated 
 * tree, you need to tell it to "Refresh Trees," which is located under 
 * the "File" menu. 
 *
 * <P>In the display, a '*' at the beginning of each object's name means it
 * implements interface Accessible.
 *
 * <P>For now, it is meant to run in the same Java Virtual Machine as 
 * the application it is accessing.  To do this, you need to make sure
 * there is an "AWT.AutoLoadClasses=" line in the file,
 * $JDKHOME/lib/awt.properties.  For example, you would add the following 
 * line in awt.properties to make this thread run with the application:
 * <PRE>
 * AWT.AutoLoadClasses=Monkey
 * </PRE>
 * <P>Note that AWT.AutoLoadClasses can take a comma-separated list of 
 * assistive technology classes.  Note also that you need to make sure 
 * the classes for the assistive technologies are in your CLASSPATH
 * environment variable.
 *
 * <P>Furthermore, this class is dynamically loaded and a single instance
 * is created by the event queue class (see EventQueueMonitor) in a separate 
 * thread.  For this to work properly, the constructor method needs to 
 * do all the work.
 *
 * <P>For more information on the event delegation model, see the event
 * delegation model specification that comes with the JDK 1.1.1 documentation.
 *
 * <p>NOTE:  This is a preliminary draft.  The methods and name may change 
 * in future beta releases.
 *
 * @see EventQueueMonitor
 *
 * @version     1.6 09/26/97 14:12:24
 * @author      Willie Walker
 */
public class Monkey implements TreeSelectionListener, TreeExpansionListener
{
    public static int WIDTH = 600;
    public static int HEIGHT = 400;
    public JFrame frame;
    public JTree componentTree; 
    public JScrollPane componentScrollPane;
    public JTree accessibleTree;
    public JScrollPane accessibleScrollPane;
    Component lastComponent = null;
    Accessible lastAccessible = null;

    JMenuBar createMenuBar() {
	JMenuBar menuBar = new JMenuBar();
	JMenuItem mi;

	// File Menu
	JMenu file = (JMenu) menuBar.add(new JMenu("File"));
        mi = (JMenuItem) file.add(new JMenuItem("Refresh Trees"));
	mi.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		componentScrollPane.getViewport().remove(componentTree);
		accessibleScrollPane.getViewport().remove(accessibleTree);
                componentTree = createComponentTree();
                accessibleTree = createAccessibleTree();
		componentScrollPane.getViewport().add(componentTree);
		accessibleScrollPane.getViewport().add(accessibleTree);
   	        frame.repaint();
	    }
	});

        file.add(new JSeparator());
        mi = (JMenuItem) file.add(new JMenuItem("Exit"));
	mi.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		System.exit(0);
	    }
	});

	return menuBar;
    }

    public void addComponentNodes(Component c, ComponentNode root) {
        String name;
        ComponentNode me;
        me = new ComponentNode(c);
        root.add(me);
        if (c instanceof Container) {
            int count = ((Container) c).getComponentCount();
            for (int i = 0; i < count; i++) {
                addComponentNodes(((Container) c).getComponent(i), me);
            }
        }
    }

    public void addAccessibleNodes(Accessible a, AccessibleNode root) {
        AccessibleNode me;
        me = new AccessibleNode(a);
        root.add(me);
	if (me.accessible.getAccessibleParent() != root.accessible) {
	    System.out.println("OOPS.  Accessible parent of " + me
		    + " should be " + root + " but it is " +
		    me.accessible.getAccessibleParent());
	}
        int count = a.getAccessibleChildrenCount();
        for (int i = 0; i < count; i++) {
            addAccessibleNodes(a.getAccessibleChild(i), me);
        }
    }

    public JTree createComponentTree() {
        JTree t;
        Window[] wins = EventQueueMonitor.getTopLevelWindows();
	ComponentNode root = new ComponentNode("Component Tree");
        for (int i = 0; i < wins.length; i++) {
            addComponentNodes(wins[i], root);
        }        
	t = new JTree(root);
	t.addTreeSelectionListener(this);
	t.addTreeExpansionListener(this);
	return t;
    }

    public JTree createAccessibleTree() {
	JTree t;
        Window[] wins = EventQueueMonitor.getTopLevelWindows();

	AccessibleNode root = new AccessibleNode("Accessible Tree");
        for (int i = 0; i < wins.length; i++) {
            if (wins[i] instanceof Accessible) {
                addAccessibleNodes((Accessible) wins[i], root);
	    }
        }        

	t = new JTree(root);
	t.addTreeSelectionListener(this);
	t.addTreeExpansionListener(this);
	return t;
    }

    public Monkey() {
	WindowListener l = new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {System.exit(0);}
	};

        frame = new JFrame("Monkey");
	frame.setLayout(new BorderLayout());
	frame.addWindowListener(l);
	frame.add("North", createMenuBar());

        JSplitPane pane = new JSplitPane();
        //pane.setLayout(new GridLayout(1,2));
        frame.add("Center", pane);

        componentTree = createComponentTree();
	componentScrollPane = new JScrollPane();
	componentScrollPane.getViewport().add(componentTree);
	pane.setLeftComponent(componentScrollPane);

        accessibleTree = createAccessibleTree();
	accessibleScrollPane = new JScrollPane();
	accessibleScrollPane.getViewport().add(accessibleTree);
	pane.setRightComponent(accessibleScrollPane);

	frame.setSize(WIDTH, HEIGHT);
	frame.show();
	frame.validate();
	frame.repaint();
    }

    public void showComponentBounds(Component c) {
        Graphics g;
        Rectangle b;

	if (c != null) {
            b = c.getBounds();
	    Container parent = c.getParent();
	    if (parent != null) {
                g = parent.getGraphics();
        	if (g != null) {
                    g.setXORMode(Color.pink);
                    g.setColor(Color.black);
                    g.fillRect(b.x, b.y, b.width, b.height);
	        }
            } else {
                g = c.getGraphics();
        	if (g != null) {
                    g.setXORMode(Color.red);
                    g.setColor(Color.black);
                    g.fillRect(0, 0, b.width, b.height);
	        }
            }
        }
    }

    public void showAccessibleBounds(Accessible a) {
	Object parent;
        Component c;
        Graphics g;
        Rectangle b;

	if (a != null) {
            b = a.getBounds();
            // This little hack is to get the graphics context
            // of the actual parent rather than the accessible
            // parent if we can.
	    if (a instanceof Component) {
		parent = ((Component) a).getParent();
	    } else {
   	        parent = a.getAccessibleParent();
	    }
	    if (parent instanceof Component) {
                g = ((Component) parent).getGraphics();
        	if (g != null) {
                    g.setXORMode(Color.pink);
                    g.setColor(Color.black);
                    g.fillRect(b.x, b.y, b.width, b.height);
	        }
            } else if (a instanceof Component) {
                g = ((Component) a).getGraphics();
        	if (g != null) {
                    g.setXORMode(Color.red);
                    g.setColor(Color.black);
                    g.fillRect(0, 0, b.width, b.height);
	        }
            }

        }
    }

    public void valueChanged(TreeSelectionEvent e) {
	Object[] path = e.getPath().getPath();

        // Erase
	showComponentBounds(lastComponent);
	lastComponent = null;
	showAccessibleBounds(lastAccessible);
	lastAccessible = null;

        // Draw
        if (e.getSource() == componentTree) {
            lastComponent = ((ComponentNode) path[path.length - 1]).component;
	    showComponentBounds(lastComponent);
	} else if (e.getSource() == accessibleTree) {
	    lastAccessible = ((AccessibleNode) path[path.length - 1]).accessible;
	    showAccessibleBounds(lastAccessible);
	}
    }

    public void treeCollapsed(TreeExpansionEvent e) {
    }

    public void treeExpanded(TreeExpansionEvent e) {
	Object[] path = e.getPath().getPath();
	//System.out.println("Tree Expanded:  " + path[path.length - 1]);
    }

    public static void main(String s[]) {

        String vers = System.getProperty("java.version");
        if (vers.compareTo("1.1.2") < 0) {
            System.out.println("!!!WARNING: Monkey must be run with a " +
                               "1.1.2 or higher version VM!!!");
        }

	new Monkey();
    }
}
